package task.defaultcandidatequalifier;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.AnnotationConfigApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Import;
import org.springframework.stereotype.Component;

public class DefaultCandidateQualifierTests {

	@Test
	void autowireByConstructorQualifierWithDefaultCandidateFalse() {
		load(AutowireByConstructorQualifierConfig.class);
	}

	@Test
	void autowireByMagicSetterQualifierWithDefaultCandidateFalse() {
		load(AutowireBySetterQualifierConfig.class);
	}

	private ApplicationContext load(Class<?>... additionalClasses) {
		AnnotationConfigApplicationContext context = new AnnotationConfigApplicationContext();
		context.register(Config.class);
		context.register(additionalClasses);
		context.refresh();
		return context;
	}

	@Configuration(proxyBeanMethods = false)
	static class Config {

		@Bean
		SomeService someService() {
			return new SomeService();
		}

		@Bean(defaultCandidate = false)
		@Qualifier("test")
		SomeService someService1() {
			return new SomeService();
		}

	}

	@Import(AutowireByConstructorQualifier.class)
	static class AutowireByConstructorQualifierConfig {

	}

	@Import(AutowireBySetterQualifier.class)
	static class AutowireBySetterQualifierConfig {

	}

	@Component
	static class AutowireByConstructorQualifier {

		private final SomeService someService;

		AutowireByConstructorQualifier(@Qualifier("test") SomeService someService) {
			this.someService = someService;
		}

	}

	@Component
	static class AutowireBySetterQualifier {

		private SomeService someService;

		@Qualifier("test")
		@Autowired
		public void setSomeService(@Qualifier("test") SomeService someService) {
			this.someService = someService;
		}

	}

	static class SomeService {

	}

}
